@@ -336,10 +336,7 @@ class Dct(Dict):
     """Dictionary with nicer formatting and dot notation access."""
 
     def __getattr__(self, key):
-        try:
-            return self[key]
-        except KeyError as exp:
-            raise AttributeError('Key ' + str(key) + ' does not exist') from exp
+        return self[key]
 
     def __setattr__(self, key, val):
         self[key] = val
